@@ -4,7 +4,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2011 Pascal Buchbinder
+ * Copyright (C) 2007-2012 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -29,8 +29,8 @@
 /* ----------------------------------
  * version info
  * ---------------------------------- */
-static const char man_version[] = "9.76";
-static const char man_date[] = "07 December 2011";
+static const char man_version[] = "10.3";
+static const char man_date[] = "February 2012";
 
 /* ----------------------------------
  * definitions
@@ -53,7 +53,7 @@ typedef struct qs_event_st {
 /* ----------------------------------
  * functions
  * ---------------------------------- */
-char *qs_CMD(char *cmd);
+char *qs_CMD(const char *cmd);
 void qs_man_print(int man, const char *fmt, ...);
 void qs_man_println(int man, const char *fmt, ...);
 
