@@ -1,9 +1,11 @@
+/* -*-mode: c; indent-tabs-mode: nil; c-basic-offset: 2; -*-
+ */
 /**
  * Command line execution utility for the quality of service module mod_qos.
  *
  * See http://opensource.adnovum.ch/mod_qos/ for further details.
  *
- * Copyright (C) 2011 Pascal Buchbinder
+ * Copyright (C) 2011-2012 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsexec.c,v 1.8 2011/11/01 22:11:13 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsexec.c,v 1.12 2012/02/09 21:03:44 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
@@ -56,6 +58,9 @@ static const char revision[] = "$Id: qsexec.c,v 1.8 2011/11/01 22:11:13 pbuchbin
 #endif
 #define MAX_REG_MATCH 10
 
+/* same as APR_SIZE_MAX which doesn't appear until APR 1.3 */
+#define QSUTIL_SIZE_MAX (~((apr_size_t)0))
+
 typedef struct {
     int rm_so;
     int rm_eo;
@@ -134,7 +139,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsfilter2(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsfilter2(1), qsgeo(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -177,6 +182,10 @@ char *qs_pregsub(apr_pool_t *pool, const char *input,
       len++;
     }
     else if (no < nmatch && pmatch[no].rm_so < pmatch[no].rm_eo) {
+      if(QSUTIL_SIZE_MAX - len <= pmatch[no].rm_eo - pmatch[no].rm_so) {
+        fprintf(stderr, "ERROR, integer overflow or out of memory condition");
+        return NULL;
+      }
       len += pmatch[no].rm_eo - pmatch[no].rm_so;
     }
     
@@ -194,7 +203,7 @@ char *qs_pregsub(apr_pool_t *pool, const char *input,
 
     if (no > 9) {                /* Ordinary character. */
       if (c == '\\' && (*src == '$' || *src == '&'))
-	c = *src++;
+        c = *src++;
       *dst++ = c;
     }
     else if (no < nmatch && pmatch[no].rm_so < pmatch[no].rm_eo) {
@@ -287,12 +296,12 @@ int main(int argc, const char * const argv[]) {
     } else if(strcmp(*argv,"-c") == 0) {
       if (--argc >= 1) {
 	clearpattern = *(++argv);
+	if (argc >=1 && *argv[0] != '-') {
+	  clearcommand = *(++argv);
+	  argc--;
+	}
       }
-      if (argc > 2 && *argv[0] != '-') {
-	clearcommand = *(++argv);
-	--argc;
-      }
-    } else if(strcmp(*argv,"-t") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"-t") == 0) {
       if (--argc >= 1) {
 	char *str = apr_pstrdup(pool, *(++argv));
 	char *tme = strchr(str, ':');
@@ -309,15 +318,15 @@ int main(int argc, const char * const argv[]) {
 	  exit(1);
 	}
       }
-    } else if(strcmp(*argv,"-p") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"-p") == 0) {
       pass = 1;
-    } else if(strcmp(*argv,"-h") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"-h") == 0) {
       usage(cmd, 0);
-    } else if(strcmp(*argv,"-?") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"-?") == 0) {
       usage(cmd, 0);
-    } else if(strcmp(*argv,"-help") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"-help") == 0) {
       usage(cmd, 0);
-    } else if(strcmp(*argv,"--man") == 0) {
+    } else if(argc >= 1 && strcmp(*argv,"--man") == 0) {
       usage(cmd, 1);
     } else {
       command = *argv;
@@ -380,7 +389,7 @@ int main(int argc, const char * const argv[]) {
 	if(!replaced) {
 	  fprintf(stderr, "[%s]: ERROR, failed to substitute submatches '%s' in (%s)\n", cmd, clearcommand, line);
 	} else {
-	  system(replaced);
+	  int rc = system(replaced);
 	}
 	executed = 0;
       }
@@ -395,7 +404,7 @@ int main(int argc, const char * const argv[]) {
 	}
 	if(counter >= threshold) {
 	  if(countertime + sec >= time(NULL)) {
-	    system(replaced);
+	    int rc = system(replaced);
 	    executed = 1;
 	  }
 	  countertime = 0;
