@@ -1,9 +1,11 @@
+/* -*-mode: c; indent-tabs-mode: nil; c-basic-offset: 2; -*-
+ */
 /**
  * Filter utility for the quality of service module mod_qos.
  *
  * See http://opensource.adnovum.ch/mod_qos/ for further details.
  *
- * Copyright (C) 2011 Pascal Buchbinder
+ * Copyright (C) 2011-2012 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +27,7 @@
  *
  */
 
-static const char revision[] = "$Id: qsgrep.c,v 1.3 2011/11/01 22:11:13 pbuchbinder Exp $";
+static const char revision[] = "$Id: qsgrep.c,v 1.6 2012/02/09 21:01:38 pbuchbinder Exp $";
 
 /* system */
 #include <stdio.h>
@@ -53,6 +55,8 @@ static const char revision[] = "$Id: qsgrep.c,v 1.3 2011/11/01 22:11:13 pbuchbin
 #define POSIX_MALLOC_THRESHOLD (10)
 #endif
 #define MAX_REG_MATCH 10
+/* same as APR_SIZE_MAX which doesn't appear until APR 1.3 */
+#define QSUTIL_SIZE_MAX (~((apr_size_t)0))
 
 typedef struct {
     int rm_so;
@@ -117,7 +121,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsexec(1), qsfilter2(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsexec(1), qsfilter2(1), qsgeo(1), qslog(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -160,6 +164,10 @@ char *qs_pregsub(apr_pool_t *pool, const char *input,
       len++;
     }
     else if (no < nmatch && pmatch[no].rm_so < pmatch[no].rm_eo) {
+      if(QSUTIL_SIZE_MAX - len <= pmatch[no].rm_eo - pmatch[no].rm_so) {
+        fprintf(stderr, "ERROR, integer overflow or out of memory condition");
+        return NULL;
+      }
       len += pmatch[no].rm_eo - pmatch[no].rm_so;
     }
     
@@ -226,6 +234,7 @@ int qs_regexec(pcre *preg, const char *string,
 }
 
 int main(int argc, const char * const argv[]) {
+  int rc;
   int nr = 0;
   char line[32768];
   FILE *file = 0;
@@ -278,6 +287,8 @@ int main(int argc, const char * const argv[]) {
     usage(cmd, 0);
   }
 
+  rc = nice(10);
+
   preg = pcre_compile(pattern, PCRE_DOTALL, &errptr, &erroffset, NULL);
   if(!preg) {
     fprintf(stderr, "ERROR, could not compile '%s' at position %d, reason: %s\n",
