@@ -6,7 +6,7 @@
  * See http://opensource.adnovum.ch/mod_qos/ for further
  * details.
  *
- * Copyright (C) 2007-2011 Pascal Buchbinder
+ * Copyright (C) 2007-2012 Pascal Buchbinder
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -25,7 +25,7 @@
  *
  */
 
-static const char revision[] = "$Id: qslog.c,v 1.26 2011/12/02 19:36:54 pbuchbinder Exp $";
+static const char revision[] = "$Id: qslog.c,v 1.40 2012/02/14 19:32:30 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <string.h>
@@ -41,6 +41,11 @@ static const char revision[] = "$Id: qslog.c,v 1.26 2011/12/02 19:36:54 pbuchbin
 #include <regex.h>
 #include <time.h>
 
+/* apr */
+#include <apr.h>
+#include <apr_portable.h>
+#include <apr_support.h>
+
 #include "qs_util.h"
 
 /* ----------------------------------
@@ -48,46 +53,81 @@ static const char revision[] = "$Id: qslog.c,v 1.26 2011/12/02 19:36:54 pbuchbin
  * ---------------------------------- */
 #define ACTIVE_TIME 600 /* how long is a client "active" (ip addresses seen in the log) */
 #define LOG_INTERVAL 60 /* log interval ist 60 sec, don't change this value */
+#define LOG_DET ".detailed"
+#define RULE_DELIM ':'
+#define MAX_CLIENT_ENTRIES 25000
 
 /* ----------------------------------
  * structures
  * ---------------------------------- */
 
+typedef struct {
+  long request_count;
+  long error_count;
+  long long duration;
+  long duration_0;
+  long duration_1;
+  long duration_2;
+  long duration_3;
+  long duration_4;
+  long duration_5;
+  long duration_6;
+  long status_1;
+  long status_2;
+  long status_3;
+  long status_4;
+  long status_5;
+} client_rec_t;
+
+typedef struct stat_rec_st {
+  // id
+  char *id;
+  regex_t preg;
+  struct stat_rec_st *next;
+
+  // counters
+  long line_count;
+  long long i_byte_count;
+  long long byte_count;
+  long duration_count;
+  long duration_0;
+  long duration_1;
+  long duration_2;
+  long duration_3;
+  long duration_4;
+  long duration_5;
+  long duration_6;
+  long connections;
+  
+  long status_1;
+  long status_2;
+  long status_3;
+  long status_4;
+  long status_5;
+  
+  long qos_v;
+  long qos_s;
+  long qos_d;
+  long qos_k;
+  long qos_t;
+  long qos_l;
+  long qos_ser;
+} stat_rec_t;
+
 /* ----------------------------------
  * global stat counter
  * ---------------------------------- */
-static long m_line_count = 0;
-static long long m_i_byte_count = -1;
-static long long m_byte_count = 0;
-static long m_duration_count = 0;
-static long m_duration_0 = 0;
-static long m_duration_1 = 0;
-static long m_duration_2 = 0;
-static long m_duration_3 = 0;
-static long m_duration_4 = 0;
-static long m_duration_5 = 0;
-static long m_duration_6 = 0;
-static long m_connections = -1;
-
-static long m_status_1 = 0;
-static long m_status_2 = 0;
-static long m_status_3 = 0;
-static long m_status_4 = 0;
-static long m_status_5 = 0;
-
-static long m_qos_v = 0;
-static long m_qos_s = 0;
-static long m_qos_d = 0;
-static long m_qos_k = 0;
-static long m_qos_t = 0;
-static long m_qos_l = 0;
-static long m_qos_ser = 0;
+static stat_rec_t* m_stat_rec;
+static stat_rec_t* m_stat_sub = NULL;
 
 static qs_event_t *m_ip_list = NULL;
 static qs_event_t *m_user_list = NULL;
+
 /* output file */
 static FILE *m_f = NULL;
+static FILE *m_f2 = NULL;
 static char  m_file_name[MAX_LINE];
+static char  m_file_name2[MAX_LINE];
 static int   m_rotate = 0;
 /* regex to search the time string */
 static regex_t m_trx;
@@ -97,6 +137,9 @@ static int   m_offline = 0;
 static int   m_offline_data = 0;
 static char  m_date_str[MAX_LINE];
 static int   m_mem = 0;
+static apr_table_t *m_client_entries = NULL;
+static int   m_max_client_entries = 0;
+static int   m_offline_count = 0;
 /* debug */
 static long  m_lines = 0;
 static int   m_verbose = 0;
@@ -122,13 +165,15 @@ static char *skipElement(const char* line) {
   char delim = p[0];
   if(delim == '\'' || delim == '\"') {
     p++;
-    while(p[0] != delim && p[0] != 0 && p[-1] != '\\') {
+    // read while we found an '" '" which is not escaped
+    while(p[0] != 0 &&
+	  !(p[0] == delim && p[-1] != '\\' && (p[1] == '\0' || p[1] == ' '))) {
       p++;
     }
     p++;
   } else {
     char *eq = NULL;
-    if(m_offline) {
+    if(m_offline || m_offline_count) {
       // offline mode: check for <name>='<value>' entry
       eq = strstr(p, "='");
       if(eq && (eq - p) < 10) {
@@ -227,11 +272,12 @@ static void getFreeMem(char *buf, int sz) {
     char vmstat[] = "/usr/bin/vmstat";
     struct stat attr;
     if(stat(vmstat, &attr) == 0) {
+      int rc;
       char command[1024];
       char outfile[1024];
       snprintf(outfile, sizeof(outfile), "/tmp/qslog.%d", getpid());
       snprintf(command, sizeof(command), "%s 1 2 1>%s", vmstat, outfile);
-      system(command);
+      rc = system(command);
       f = fopen(outfile, "r");
       if(f) {
         char line[MAX_LINE];
@@ -272,35 +318,30 @@ static void getFreeMem(char *buf, int sz) {
 #define NBIS "ib/s"
 #define NAV "av"
 
-/*
- * writes all stat data to the out file
- * an resets all counters
- */
-static void printAndResetStat(char *timeStr) {
-  double av[1];
-  char mem[256];
+static void printStat2File(FILE *f, char *timeStr, stat_rec_t *stat_rec, int offline, int main,
+			   double *av, const char *mem) {
   char bis[256];
   char esco[256];
+  char ip[256];
+  char usr[256];
   bis[0] = '\0';
   esco[0] = '\0';
-  if(m_i_byte_count != -1) {
-    sprintf(bis, NBIS";%lld;", m_i_byte_count/LOG_INTERVAL);
+  if(stat_rec->i_byte_count != -1) {
+    sprintf(bis, NBIS";%lld;", stat_rec->i_byte_count/LOG_INTERVAL);
   }
-  if(m_connections != -1) {
-    sprintf(esco, "esco;%ld;", m_connections);
+  if(main && stat_rec->connections != -1) {
+    sprintf(esco, "esco;%ld;", stat_rec->connections);
   }
-  if(!m_offline) {
-    getloadavg(av, 1);
-    if(m_mem) {
-      getFreeMem(mem, sizeof(mem));
-    } else {
-      mem[0] = '\0';
-    }
+  if(main) {
+    sprintf(ip, "ip;%ld;", qs_countEvent(&m_ip_list));
+    sprintf(usr, "usr;%ld;", qs_countEvent(&m_user_list));
   } else {
-    mem[0] = '\0';
+    ip[0] = '\0';
+    usr[0] = '\0';
   }
-  qs_csLock();
-  fprintf(m_f, "%s;"
+
+  fprintf(f, "%s;"
+	  "%s"
           NRS";%ld;"
 	  "req;%ld;"
           NBS";%lld;"
@@ -319,8 +360,8 @@ static void printAndResetStat(char *timeStr) {
           "4s;%ld;"
           "5s;%ld;"
           ">5s;%ld;"
-	  "ip;%ld;"
-	  "usr;%ld;"
+	  "%s"
+	  "%s"
 	  "qV;%ld;"
 	  "qS;%ld;"
 	  "qD;%ld;"
@@ -330,72 +371,332 @@ static void printAndResetStat(char *timeStr) {
 	  "qs;%ld;"
 	  ,
 	  timeStr,
-          m_line_count/LOG_INTERVAL,
-	  m_line_count,
-          m_byte_count/LOG_INTERVAL,
+	  main ? "" : stat_rec->id,
+          stat_rec->line_count/LOG_INTERVAL,
+	  stat_rec->line_count,
+          stat_rec->byte_count/LOG_INTERVAL,
 	  bis,
 	  esco,
-	  m_status_1,
-	  m_status_2,
-	  m_status_3,
-	  m_status_4,
-	  m_status_5,
-          m_duration_count/(m_line_count == 0 ? 1 : m_line_count),
-          m_duration_0,
-          m_duration_1,
-          m_duration_2,
-          m_duration_3,
-          m_duration_4,
-          m_duration_5,
-          m_duration_6,
-          qs_countEvent(&m_ip_list),
-          qs_countEvent(&m_user_list),
-	  m_qos_v,
-	  m_qos_s,
-	  m_qos_d,
-	  m_qos_k,
-	  m_qos_t,
-	  m_qos_l,
-	  m_qos_ser
+	  stat_rec->status_1,
+	  stat_rec->status_2,
+	  stat_rec->status_3,
+	  stat_rec->status_4,
+	  stat_rec->status_5,
+          stat_rec->duration_count/(stat_rec->line_count == 0 ? 1 : stat_rec->line_count),
+          stat_rec->duration_0,
+          stat_rec->duration_1,
+          stat_rec->duration_2,
+          stat_rec->duration_3,
+          stat_rec->duration_4,
+          stat_rec->duration_5,
+          stat_rec->duration_6,
+	  ip,
+	  usr,
+	  stat_rec->qos_v,
+	  stat_rec->qos_s,
+	  stat_rec->qos_d,
+	  stat_rec->qos_k,
+	  stat_rec->qos_t,
+	  stat_rec->qos_l,
+	  stat_rec->qos_ser
           );
-  m_line_count = 0;
-  m_byte_count = 0;
-  if(m_i_byte_count != -1) {
-    m_i_byte_count = 0;
+  stat_rec->line_count = 0;
+  stat_rec->byte_count = 0;
+  if(stat_rec->i_byte_count != -1) {
+    stat_rec->i_byte_count = 0;
   }
-  if(m_connections != -1) {
-    m_connections = 0;
+  if(main && (stat_rec->connections != -1)) {
+    stat_rec->connections = 0;
   }
-  m_status_1 = 0;
-  m_status_2 = 0;
-  m_status_3 = 0;
-  m_status_4 = 0;
-  m_status_5 = 0;
-  m_duration_count = 0;
-  m_duration_0 = 0;
-  m_duration_1 = 0;
-  m_duration_2 = 0;
-  m_duration_3 = 0;
-  m_duration_4 = 0;
-  m_duration_5 = 0;
-  m_duration_6 = 0;
-  m_qos_v = 0;
-  m_qos_s = 0;
-  m_qos_d = 0;
-  m_qos_k = 0;
-  m_qos_t = 0;
-  m_qos_l = 0;
-  m_qos_ser = 0;
+  stat_rec->status_1 = 0;
+  stat_rec->status_2 = 0;
+  stat_rec->status_3 = 0;
+  stat_rec->status_4 = 0;
+  stat_rec->status_5 = 0;
+  stat_rec->duration_count = 0;
+  stat_rec->duration_0 = 0;
+  stat_rec->duration_1 = 0;
+  stat_rec->duration_2 = 0;
+  stat_rec->duration_3 = 0;
+  stat_rec->duration_4 = 0;
+  stat_rec->duration_5 = 0;
+  stat_rec->duration_6 = 0;
+  stat_rec->qos_v = 0;
+  stat_rec->qos_s = 0;
+  stat_rec->qos_d = 0;
+  stat_rec->qos_k = 0;
+  stat_rec->qos_t = 0;
+  stat_rec->qos_l = 0;
+  stat_rec->qos_ser = 0;
+  if(main) {
+    if(!offline) {
+      fprintf(f, "sl;%.2f;m;%s",
+	      av[0], mem[0] ? mem : "-");
+    } else {
+      fprintf(f, "sl;-;m;-");
+      m_offline_data = 0;
+    }
+  }
+  fprintf(f, "\n");
+}
+
+static stat_rec_t *createRec(const char *id, const char *pattern) {
+  stat_rec_t *rec = calloc(sizeof(stat_rec_t), 1);
+  rec->id = calloc(strlen(id)+2, 1);
+  sprintf(rec->id, "%s;", id);
+  rec->id[strlen(id)+1] = '\0';
+  if(regcomp(&rec->preg, pattern, REG_EXTENDED)) {
+    qerror("failed to compile pattern %s", pattern);
+    exit(1);
+  }
+  rec->next = NULL;
+
+  rec->line_count = 0;
+  rec->i_byte_count = -1;
+  rec->byte_count = 0;
+  rec->duration_count = 0;
+  rec->duration_0 = 0;
+  rec->duration_1 = 0;
+  rec->duration_2 = 0;
+  rec->duration_3 = 0;
+  rec->duration_4 = 0;
+  rec->duration_5 = 0;
+  rec->duration_6 = 0;
+  rec->connections = -1;
+
+  rec->status_1 = 0;
+  rec->status_2 = 0;
+  rec->status_3 = 0;
+  rec->status_4 = 0;
+  rec->status_5 = 0;
+
+  rec->qos_v = 0;
+  rec->qos_s = 0;
+  rec->qos_d = 0;
+  rec->qos_k = 0;
+  rec->qos_t = 0;
+  rec->qos_l = 0;
+  rec->qos_ser = 0;
+  return rec;
+}
+
+static stat_rec_t *getRec(const char *value) {
+  regmatch_t ma[1];
+  int len = 0;
+  stat_rec_t *r = m_stat_sub;
+  stat_rec_t *rec = NULL;
+  while(r) {
+    if(regexec(&r->preg, value, 1, ma, 0) == 0) {
+      int l = ma[0].rm_eo - ma[0].rm_so + 1;
+      if(l > len) {
+	// longest match
+	len = l;
+	rec = r;
+      }
+    }
+    r = r->next;
+  }
+  return rec;
+}
+
+/*
+ * writes all stat data to the out file
+ * an resets all counters
+ */
+static void printAndResetStat(char *timeStr) {
+  stat_rec_t *r = m_stat_sub;
+  double av[1];
+  char mem[256];
   if(!m_offline) {
-    fprintf(m_f, "sl;%.2f;m;%s",
-	    av[0], mem[0] ? mem : "-");
+    getloadavg(av, 1);
+    if(m_mem) {
+      getFreeMem(mem, sizeof(mem));
+    } else {
+      mem[0] = '\0';
+    }
   } else {
-    fprintf(m_f, "sl;-;m;-");
-    m_offline_data = 0;
+    mem[0] = '\0';
+  }
+  qs_csLock();
+  printStat2File(m_f, timeStr, m_stat_rec, m_offline, 1, av, mem);
+  while(r) {
+    printStat2File(m_f2, timeStr, r, m_offline, 0, av, mem);
+    r = r->next;
   }
-  fprintf(m_f, "\n");
   qs_csUnLock();
   fflush(m_f);
+  if(m_f2) {
+    fflush(m_f2);
+  }
+}
+
+static void updateClient(stat_rec_t *rec, char *T, char *t, char *D, char *S,
+			 char *BI, char *B, char *R, char *I, char *U, char *Q, char *k, char *C,
+			 long tme) {
+  long long start;
+  long long end;
+  struct timeval tv;
+  client_rec_t *client_rec;
+  const char *id = I; // ip
+  if(id == NULL) {
+    id = U; // user
+  }
+  if(id == NULL) {
+    return;
+  }
+  client_rec = (client_rec_t *)apr_table_get(m_client_entries, id);
+  if(client_rec == NULL) {
+    char *tid;
+    if(apr_table_elts(m_client_entries)->nelts >= MAX_CLIENT_ENTRIES) {
+      // limitation: speed (table to big) and memory
+      if(!m_max_client_entries) {
+	fprintf(stderr, "\nreached max client enries (%d)\n", MAX_CLIENT_ENTRIES);
+	m_max_client_entries = 1;
+      }
+      return;
+    }
+    tid = calloc(strlen(id)+1, 1);
+    client_rec = calloc(sizeof(client_rec_t), 1);
+    strcpy(tid, id);
+    tid[strlen(id)] = '\0';
+    client_rec->request_count = 0;
+    client_rec->error_count = 0;
+    client_rec->duration = 0;
+    client_rec->duration_0 = 0;
+    client_rec->duration_1 = 0;
+    client_rec->duration_2 = 0;
+    client_rec->duration_3 = 0;
+    client_rec->duration_4 = 0;
+    client_rec->duration_5 = 0;
+    client_rec->duration_6 = 0;
+    client_rec->status_1 = 0;
+    client_rec->status_2 = 0;
+    client_rec->status_3 = 0;
+    client_rec->status_4 = 0;
+    client_rec->status_5 = 0;
+    apr_table_setn(m_client_entries, tid, (char *)client_rec);
+  }
+  client_rec->request_count++;
+  client_rec->duration += tme;
+  if(tme < 1) {
+    client_rec->duration_0++;
+  } else if(tme == 1) {
+    client_rec->duration_1++;
+  } else if(tme == 2) {
+    client_rec->duration_2++;
+  } else if(tme == 3) {
+    client_rec->duration_3++;
+  } else if(tme == 4) {
+    client_rec->duration_4++;
+  } else if(tme == 5) {
+    client_rec->duration_5++;
+  } else {
+    client_rec->duration_6++;
+  }
+  if(S != NULL) {
+    if(strcmp(S, "200") != 0 && strcmp(S, "304") != 0 && strcmp(S, "302") != 0) {
+      client_rec->error_count++;
+    }
+    if(S[0] == '1') {
+      client_rec->status_1++;
+    } else if(S[0] == '1') {
+      client_rec->status_1++;
+    } else if(S[0] == '2') {
+      client_rec->status_2++;
+    } else if(S[0] == '3') {
+      client_rec->status_3++;
+    } else if(S[0] == '4') {
+      client_rec->status_4++;
+    } else if(S[0] == '5') {
+      client_rec->status_5++;
+    }
+  }
+  return;
+}
+
+static void updateRec(stat_rec_t *rec, char *T, char *t, char *D, char *S,
+		      char *BI, char *B, char *R, char *I, char *U, char *Q, char *k, char *C,
+		      long tme) {
+  if(Q != NULL) {
+    if(strchr(Q, 'V') != NULL) {
+      rec->qos_v++;
+    }
+    if(strchr(Q, 'S') != NULL) {
+      rec->qos_s++;
+    }
+    if(strchr(Q, 'D') != NULL) {
+      rec->qos_d++;
+    }
+    if(strchr(Q, 'K') != NULL) {
+      rec->qos_k++;
+    }
+    if(strchr(Q, 'T') != NULL) {
+      rec->qos_t++;
+    }
+    if(strchr(Q, 'L') != NULL) {
+      rec->qos_l++;
+    }
+    if(strchr(Q, 's') != NULL) {
+      rec->qos_ser++;
+    }
+  }
+  if(I != NULL) {
+    /* update/store client IP */
+    qs_insertEvent(&m_ip_list, I);
+  }
+  if(U != NULL) {
+    /* update/store user */
+    qs_insertEvent(&m_user_list, U);
+  }
+  if(B != NULL) {
+    /* transferred bytes */
+    rec->byte_count += atoi(B);
+  }
+  if(BI != NULL) {
+    /* transferred bytes */
+    rec->i_byte_count += atoi(BI);
+  }
+  if(k != NULL) {
+    if(k[0] == '0' && k[1] == '\0') {
+      rec->connections++;
+    }
+  }
+  if(S != NULL) {
+    if(S[0] == '1') {
+      rec->status_1++;
+    } else if(S[0] == '1') {
+      rec->status_1++;
+    } else if(S[0] == '2') {
+      rec->status_2++;
+    } else if(S[0] == '3') {
+      rec->status_3++;
+    } else if(S[0] == '4') {
+      rec->status_4++;
+    } else if(S[0] == '5') {
+      rec->status_5++;
+    } 
+  }
+  if(T != NULL || t != NULL || D != NULL) {
+    /* response duration */
+    rec->duration_count += tme;
+    if(tme < 1) {
+      rec->duration_0++;
+    } else if(tme == 1) {
+      rec->duration_1++;
+    } else if(tme == 2) {
+      rec->duration_2++;
+    } else if(tme == 3) {
+      rec->duration_3++;
+    } else if(tme == 4) {
+      rec->duration_4++;
+    } else if(tme == 5) {
+      rec->duration_5++;
+    } else {
+      rec->duration_6++;
+    }
+  }
+  /* request counter */
+  rec->line_count++;
 }
 
 /*
@@ -411,6 +712,7 @@ static void printAndResetStat(char *timeStr) {
  * .         .                     .      R                                   .            T
  */
 static void updateStat(const char *cstr, char *line) {
+  stat_rec_t *rec = NULL;
   char *T = NULL; /* time */
   char *t = NULL; /* time ms */
   char *D = NULL; /* time us */
@@ -422,6 +724,7 @@ static void updateStat(const char *cstr, char *line) {
   char *U = NULL; /* user */
   char *Q = NULL; /* mod_qos event message */
   char *k = NULL; /* connections (keep alive requests = 0) */
+  char *C = NULL; /* custom patter matching the config file */
   const char *c = cstr;
   char *l = line;
   long tme;
@@ -460,6 +763,10 @@ static void updateStat(const char *cstr, char *line) {
       if(l != NULL && l[0] != '\0') {
         k = cutNext(&l);
       }
+    } else if(c[0] == 'C') {
+      if(l != NULL && l[0] != '\0') {
+        C = cutNext(&l);
+      }
     } else if(c[0] == 'R') {
       if(l != NULL && l[0] != '\0') {
         R = cutNext(&l);
@@ -486,71 +793,26 @@ static void updateStat(const char *cstr, char *line) {
     }
     c++;
   }
+  if(C) {
+    rec = getRec(C);
+  }
 
   qs_csLock();
-  if(Q != NULL) {
-    if(strchr(Q, 'V') != NULL) {
-      m_qos_v++;
-    }
-    if(strchr(Q, 'S') != NULL) {
-      m_qos_s++;
-    }
-    if(strchr(Q, 'D') != NULL) {
-      m_qos_d++;
-    }
-    if(strchr(Q, 'K') != NULL) {
-      m_qos_k++;
-    }
-    if(strchr(Q, 'T') != NULL) {
-      m_qos_t++;
-    }
-    if(strchr(Q, 'L') != NULL) {
-      m_qos_l++;
-    }
-    if(strchr(Q, 's') != NULL) {
-      m_qos_ser++;
-    }
-  }
-  if(I != NULL) {
-    /* update/store client IP */
-    qs_insertEvent(&m_ip_list, I);
-  }
-  if(U != NULL) {
-    /* update/store user */
-    qs_insertEvent(&m_user_list, U);
-  }
   if(B != NULL) {
     /* transferred bytes */
     stripNum(&B);
-    m_byte_count = m_byte_count + atoi(B);
   }
   if(BI != NULL) {
     /* transferred bytes */
     stripNum(&BI);
-    m_i_byte_count = m_i_byte_count + atoi(BI);
   }
   if(k != NULL) {
     stripNum(&k);
-    if(k[0] == '0' && k[1] == '\0') {
-      m_connections++;
-    }
   }
   if(S != NULL) {
     stripNum(&S);
-    if(S[0] == '1') {
-      m_status_1++;
-    } else if(S[0] == '1') {
-      m_status_1++;
-    } else if(S[0] == '2') {
-      m_status_2++;
-    } else if(S[0] == '3') {
-      m_status_3++;
-    } else if(S[0] == '4') {
-      m_status_4++;
-    } else if(S[0] == '5') {
-      m_status_5++;
-    }
   }
+  tme = 0;
   if(T != NULL || t != NULL || D != NULL) {
     /* response duration */
     if(T) {
@@ -563,30 +825,19 @@ static void updateStat(const char *cstr, char *line) {
       stripNum(&D);
       tme = atol(D) / 1000000;
     }
-    m_duration_count = m_duration_count + tme;
-    if(tme < 1) {
-      m_duration_0++;
-    } else if(tme == 1) {
-      m_duration_1++;
-    } else if(tme == 2) {
-      m_duration_2++;
-    } else if(tme == 3) {
-      m_duration_3++;
-    } else if(tme == 4) {
-      m_duration_4++;
-    } else if(tme == 5) {
-      m_duration_5++;
-    } else {
-      m_duration_6++;
-    }
   }
-  /* request counter */
-  m_line_count++;
+  updateRec(m_stat_rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+  if(rec) {
+    updateRec(rec, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+  }
+  if(m_offline_count) {
+    updateClient(NULL, T, t, D, S, BI, B, R, I, U, Q, k, C, tme);
+  }
   qs_csUnLock();
 
-  if(m_offline && m_verbose) {
+  if(m_verbose && (m_offline || m_offline_count)) {
     m_lines++;
-    printf("[%ld] I=%s U=%s B=%s i=%s S=%s T=%ld Q=%s\n", m_lines,
+    printf("[%ld] I=[%s] U=[%s] B=[%s] i=[%s] S=[%s] T=[%ld] Q=[%s]\n", m_lines,
 	   I == NULL ? "(null)" : I,
 	   U == NULL ? "(null)" : U,
 	   B == NULL ? "(null)" : B,
@@ -743,7 +994,9 @@ static void readStdinOffline(const char *cstr) {
       }
       while(l_time > unitTime) {
 	snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
-	printAndResetStat(buf);
+	if(m_offline) {
+	  printAndResetStat(buf);
+	}
 	unitTime++;
 	qs_setTime(unitTime * 60);;
       }
@@ -752,7 +1005,9 @@ static void readStdinOffline(const char *cstr) {
   }
   if(m_offline_data) {
     snprintf(buf, sizeof(buf), "%s %.2ld:%.2ld:00", m_date_str, unitTime/60, unitTime%60);
-    printAndResetStat(buf);
+    if(m_offline) {
+      printAndResetStat(buf);
+    }
   }
 }
 
@@ -777,8 +1032,10 @@ static void *loggerThread(void *argv) {
       strftime(buf, sizeof(buf), "%H:%M", ptr);
       if(strcmp(buf, "23:59") == 0) {
 	char arch[MAX_LINE];
+	char arch2[MAX_LINE];
 	strftime(buf, sizeof(buf), "%Y%m%d%H%M%S", ptr);
 	snprintf(arch, sizeof(arch), "%s.%s", m_file_name, buf);
+	snprintf(arch2, sizeof(arch), "%s.%s", m_file_name2, buf);
 	if(fclose(m_f) != 0) {
 	  qerror("failed to close file '%s': %s", m_file_name, strerror(errno));
 	}
@@ -786,13 +1043,18 @@ static void *loggerThread(void *argv) {
 	  qerror("failed to move file '%s': %s", arch, strerror(errno));
 	}
 	m_f = fopen(m_file_name, "a+"); 
+	if(m_f2) {
+	  fclose(m_f2);
+	  rename(m_file_name2, arch2);
+	  m_f2 = fopen(m_file_name2, "a+"); 
+	}
       }
     }
   }
   return NULL;
 }
 
-static void usage(char *cmd, int man) {
+static void usage(const char *cmd, int man) {
   if(man) {
     //.TH [name of program] [section number] [center footer] [left footer] [center header]
     printf(".TH %s 1 \"%s\" \"mod_qos utilities %s\" \"%s man page\"\n", qs_CMD(cmd), man_date,
@@ -807,7 +1069,7 @@ static void usage(char *cmd, int man) {
   if(man) {
     printf(".SH SYNOPSIS\n");
   }
-  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p [-v]] [-x] [-u <name>] [-m]\n", man ? "" : "Usage: ", cmd);
+  qs_man_print(man, "%s%s -f <format_string> -o <out_file> [-p [-v]] [-x] [-u <name>] [-m] [-c <path>]\n", man ? "" : "Usage: ", cmd);
   printf("\n");
   if(man) {
     printf(".SH DESCRIPTION\n");
@@ -818,6 +1080,7 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, "the data from stdin. The output is written to the specified\n");
   qs_man_println(man, "file every minute and includes the following entries:\n");
   qs_man_println(man, "  - requests per second ("NRS")\n");
+  qs_man_println(man, "  - requests within the last minute (req)\n");
   qs_man_println(man, "  - bytes sent to the client per second ("NBS")\n");
   qs_man_println(man, "  - bytes received from the client per second ("NBIS")\n");
   qs_man_println(man, "  - repsonse status codes within the last minute (1xx,2xx,3xx,4xx,5xx)\n");
@@ -855,10 +1118,11 @@ static void usage(char *cmd, int man) {
   qs_man_println(man, "     i defines the received bytes (%%I)\n");
   qs_man_println(man, "     T defines the request duration (%%T)\n");
   qs_man_println(man, "     t defines the request duration in milliseconds (optionally used instead of T)\n");
-  qs_man_println(man, "     D defines the request duration in microseconds (optionally used instead of T)\n");
+  qs_man_println(man, "     D defines the request duration in microseconds (optionally used instead of T) (%%D)\n");
   qs_man_println(man, "     k defines the number of keepalive requests on the connection (%%k)\n");
   qs_man_println(man, "     U defines the user tracking id (%%{mod_qos_user_id}e)\n");
   qs_man_println(man, "     Q defines the mod_qos_ev event message (%%{mod_qos_ev}e)\n");
+  qs_man_println(man, "     C defines the element for the detailed log (-c option), e.g. \"%%U\"\n");
   qs_man_println(man, "     . defines an element to ignore (unknown string)\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -o <out_file>\n");
@@ -867,9 +1131,13 @@ static void usage(char *cmd, int man) {
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -p\n");
   if(man) printf("\n");
-  qs_man_print(man, "     Used when reading the log data from a file (cat/pipe). %s is\n", cmd);
-  qs_man_print(man, "     started using it's offline mode in order to process existing log\n");
+  qs_man_print(man, "     Used for post processing when reading the log data from a file (cat/pipe).\n");
+  qs_man_print(man, "     %s is started using it's offline mode in order to process existing log\n", cmd);
   qs_man_print(man, "     files (post processing).\n");
+  qs_man_print(man, "     The option \"-pc\" may be used alternatively if you want to gather request\n");
+  qs_man_print(man, "     information per client (identified by IP address (I) or user tracking id (U)\n");
+  qs_man_print(man, "     showing how many request each client has performed within the captured period\n");
+  qs_man_print(man, "     of time).\n");
   if(man) printf("\n.TP\n");
   qs_man_print(man, "  -v\n");
   if(man) printf("\n");
@@ -886,6 +1154,14 @@ static void usage(char *cmd, int man) {
   qs_man_print(man, "  -m\n");
   if(man) printf("\n");
   qs_man_print(man, "     Calculates free system memory every minute.\n");
+  if(man) printf("\n.TP\n");
+  qs_man_print(man, "  -c <path>\n");
+  if(man) printf("\n");
+  qs_man_print(man, "     Enables the collection of log statitics for different request types.\n");
+  qs_man_print(man, "     'path' specifies the necessary rule file. Each rule consists of a rule\n");
+  qs_man_print(man, "     identifier and a regular expression to identify a request seprarated\n");
+  qs_man_print(man, "     by a colon, e.g., 01:^(/a)|(/c). The regular expressions are matched against\n");
+  qs_man_print(man, "     the log data element which has been identified by the 'C' format character.\n");
   printf("\n");
   if(man) {
     printf(".SH EXAMPLE\n");
@@ -915,7 +1191,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsexec(1), qsfilter2(1), qsgrep(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
+    printf("qsexec(1), qsfilter2(1), qsgeo(1), qsgrep(1), qspng(1), qsrotate(1), qssign(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
@@ -928,13 +1204,51 @@ static void usage(char *cmd, int man) {
   }
 }
 
-int main(int argc, char **argv) {
-  char *config = NULL;
-  char *file = NULL;
-  char *cmd = strrchr(argv[0], '/');
-  char *username = NULL;
+static stat_rec_t *loadRule(const char *confFile) {
+  char line[MAX_LINE];
+  FILE *file = fopen(confFile, "r"); 
+  stat_rec_t *rec = NULL;
+  stat_rec_t *prev = NULL;
+  stat_rec_t *next = NULL;
+  if(file == NULL) {
+    qerror("could not open file for writing '%s': ", confFile, strerror(errno));
+    exit(1);
+  }
+  while(!qs_getLinef(line, sizeof(line), file)) {
+    char *id = line;
+    char *p = strchr(line, RULE_DELIM);
+    if(p) {
+      p[0] = '\0';
+      p++;
+      if(m_verbose) {
+	printf("load rule %s: %s\n", id, p);
+      }
+      next = createRec(id, p);
+      if(rec == NULL) {
+	rec = next;
+      }
+      if(prev) {
+	prev->next = next;
+      } else {
+	rec->next = next;
+      }
+      prev = next;
+    }
+  }
+  fclose(file);
+  return rec;
+}
+
+int main(int argc, const char *const argv[]) {
+  const char *config = NULL;
+  const char *file = NULL;
+  const char *confFile = NULL;
+  const char *cmd = strrchr(argv[0], '/');
+  const char *username = NULL;
   pthread_attr_t *tha = NULL;
   pthread_t tid;
+  m_stat_rec = createRec("", "");
+
   qs_csInitLock();
   qs_setExpiration(ACTIVE_TIME);
   if(cmd == NULL) {
@@ -950,11 +1264,11 @@ int main(int argc, char **argv) {
 	config = *(++argv);
 	if(strchr(config, 'i')) {
 	  // enable ib/s
-	  m_i_byte_count = 0;
+	  m_stat_rec->i_byte_count = 0;
 	}
 	if(strchr(config, 'k')) {
 	  // enable esco
-	  m_connections = 0;
+	  m_stat_rec->connections = 0;
 	}
       }
     } else if(strcmp(*argv,"-o") == 0) { /* this is the out file */
@@ -965,9 +1279,16 @@ int main(int argc, char **argv) {
       if (--argc >= 1) {
 	username = *(++argv);
       }
+    } else if(strcmp(*argv,"-c") == 0) { /* custom patterns (e.g. url pattern list, format: <id>':'<pattern>) */
+      if (--argc >= 1) {
+	confFile = *(++argv);
+      }
     } else if(strcmp(*argv,"-p") == 0) { /* activate offline analysis */
       m_offline = 1;
       qs_set2OfflineMode();
+    } else if(strcmp(*argv,"-pc") == 0) { /* activate offline counting analysis */
+      m_offline_count = 1;
+      qs_set2OfflineMode();
     } else if(strcmp(*argv,"-m") == 0) { /* activate memory usage */
       m_mem = 1;
     } else if(strcmp(*argv,"-v") == 0) {
@@ -989,7 +1310,67 @@ int main(int argc, char **argv) {
     argc--;
     argv++;
   }
-  /* requires at leas an output file and a format string */
+
+  if(m_offline || m_offline_count) {
+    /* init time pattern regex, std apache access log */
+    regcomp(&m_trx, 
+	    "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
+	    REG_EXTENDED);
+    /* other time patterns: "yyyy mm dd hh:mm:ss,mmm" or "yyyy mm dd hh:mm:ss.mmm" */
+    regcomp(&m_trx2, 
+	    "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
+	    REG_EXTENDED);
+  }
+
+  if(m_offline_count) {
+    int i;
+    apr_table_entry_t *entry;
+    apr_pool_t *pool;
+    int rc = nice(10);
+    if(config == NULL) usage(cmd, 0);
+    apr_app_initialize(&argc, &argv, NULL);
+    apr_pool_create(&pool, NULL);
+    m_client_entries = apr_table_make(pool, MAX_CLIENT_ENTRIES);
+    readStdinOffline(config);
+    fprintf(stdout, ".\n");
+    entry = (apr_table_entry_t *) apr_table_elts(m_client_entries)->elts;
+    m_f = stdout;
+    if(file) {
+      m_f = fopen(file, "a+");
+      if(m_f == NULL) {
+	m_f == stdout;
+      }
+    }
+    for(i = 0; i < apr_table_elts(m_client_entries)->nelts; i++) {
+      client_rec_t *client_rec = (client_rec_t *)entry[i].val;
+      fprintf(m_f, "%s;req;%ld;errors;%ld;"
+	      "1xx;%ld;2xx;%ld;3xx;%ld;4xx;%ld;5xx;%ld;"
+	      "av;%lld;<1s;%ld;1s;%ld;2s;%ld;3s;%ld;4s;%ld;5s;%ld;>5s;%ld;"
+	      "\n",
+	      entry[i].key,
+	      client_rec->request_count,
+	      client_rec->error_count,
+	      client_rec->status_1,
+	      client_rec->status_2,
+	      client_rec->status_3,
+	      client_rec->status_4,
+	      client_rec->status_5,
+	      client_rec->duration / client_rec->request_count,
+	      client_rec->duration_0,
+	      client_rec->duration_1,
+	      client_rec->duration_2,
+	      client_rec->duration_3,
+	      client_rec->duration_4,
+	      client_rec->duration_5,
+	      client_rec->duration_6);
+    }
+    if(file && m_f != stdout) {
+      fclose(m_f);
+    }
+    return 0;
+  }
+
+  /* requires at least an output file and a format string */
   if(file == NULL || config == NULL) usage(cmd, 0);
 
   if(username && getuid() == 0) {
@@ -1012,27 +1393,34 @@ int main(int argc, char **argv) {
       exit(1);
     }
   }
+
   m_f = fopen(file, "a+"); 
   if(m_f == NULL) {
-    qerror("could not open file for writing '%s': ", file, strerror(errno));
+    qerror("could not open file for writing '%s': %s", file, strerror(errno));
     exit(1);
   }
-  if(strlen(file) > (sizeof(m_file_name) - strlen(".yyyymmddHHMMSS "))) {
+  if(strlen(file) > (sizeof(m_file_name) - strlen(".yyyymmddHHMMSS  ") - strlen(LOG_DET))) {
     qerror("file name too long '%s'", file);
     exit(1);
   }
   strcpy(m_file_name, file);
 
+  if(confFile) {
+    snprintf(m_file_name2, sizeof(m_file_name2), "%s"LOG_DET, m_file_name);
+    if(strchr(config, 'C') == NULL) {
+      qerror("you need to add 'C' to the format string when enabling the pattern list (-c)");
+      exit(1);
+    }
+    m_stat_sub = loadRule(confFile);
+    m_f2 = fopen(m_file_name2, "a+");
+    if(m_f == NULL) {
+      qerror("could not open file for writing '%s': %s", m_file_name2, strerror(errno));
+      exit(1);
+    }
+  }
+
   if(m_offline) {
-    nice(10);
-    /* init time pattern regex, std apache access log */
-    regcomp(&m_trx, 
-	    "[0-9]{2}/[a-zA-Z]{3}/[0-9]{4}:[0-9]{2}:[0-9]{2}:[0-9]{2}",
-	    REG_EXTENDED);
-    /* other time patterns: yyyy mm dd hh:mm:ss,mmm */
-    regcomp(&m_trx2, 
-	    "[0-9]{4}[ -]{1}[0-9]{2}[ -]{1}[0-9]{2} [0-9]{2}:[0-9]{2}:[0-9]{2}[,.]{1}[0-9]{3}",
-	    REG_EXTENDED);
+    int rc = nice(10);
     fprintf(stderr, "[%s]: offline mode (writes to %s)\n", cmd, file);
     m_date_str[0] = '\0';
     readStdinOffline(config);
