@@ -13,6 +13,9 @@
  * as published by the Free Software Foundation; either version 2
  * of the License, or (at your option) any later version.
  *
+ * This program is released under the GPL with the additional
+ * exemption that compiling, linking, and/or using OpenSSL is allowed.
+ *
  * This program is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
@@ -25,7 +28,7 @@
  *
  */
 
-static const char revision[] = "$Id: qssign.c,v 1.19 2011/11/01 22:11:13 pbuchbinder Exp $";
+static const char revision[] = "$Id: qssign.c,v 1.21 2012/02/19 22:07:22 pbuchbinder Exp $";
 
 #include <stdio.h>
 #include <unistd.h>
@@ -554,7 +557,7 @@ static void usage(char *cmd, int man) {
   printf("\n");
   if(man) {
     printf(".SH SEE ALSO\n");
-    printf("qsexec(1), qsfilter2(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qstail(1)\n");
+    printf("qsexec(1), qsfilter2(1), qsgeo(1), qsgrep(1), qslog(1), qspng(1), qsrotate(1), qstail(1)\n");
     printf(".SH AUTHOR\n");
     printf("Pascal Buchbinder, http://opensource.adnovum.ch/mod_qos/\n");
   } else {
